@@ -10,8 +10,14 @@ from sympy.physics.secondquant import (
     ContractionAppliesOnlyToFermions
 )
 
-from sympy import (Dummy, expand, Function, I, S, simplify, sqrt, Sum,
-                   Symbol, symbols, srepr, Rational)
+from sympy.concrete.summations import Sum
+from sympy.core.function import (Function, expand)
+from sympy.core.numbers import (I, Rational)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Dummy, Symbol, symbols)
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.printing.repr import srepr
+from sympy.simplify.simplify import simplify
 
 from sympy.testing.pytest import slow, raises
 from sympy.printing.latex import latex
@@ -417,9 +423,9 @@ def test_NO():
     assert NO(Fd(a)*F(b)) == - NO(F(b)*Fd(a))
 
     no = NO(Fd(a)*F(i)*F(b)*Fd(j))
-    l1 = [ ind for ind in no.iter_q_creators() ]
+    l1 = list(no.iter_q_creators())
     assert l1 == [0, 1]
-    l2 = [ ind for ind in no.iter_q_annihilators() ]
+    l2 = list(no.iter_q_annihilators())
     assert l2 == [3, 2]
     no = NO(Fd(a)*Fd(i))
     assert no.has_q_creators == 1
