@@ -1,14 +1,22 @@
 """py.test hacks to support XFAIL/XPASS"""
 
 import sys
+import re
 import functools
 import os
 import contextlib
 import warnings
+import inspect
+import pathlib
+from typing import Any, Callable
 
 from sympy.utilities.exceptions import SymPyDeprecationWarning
+# Imported here for backwards compatibility. Note: do not import this from
+# here in library code (importing sympy.pytest in library code will break the
+# pytest integration).
+from sympy.utilities.exceptions import ignore_warnings # noqa:F401
 
-ON_TRAVIS = os.getenv('TRAVIS_BUILD_NUMBER', None)
+ON_CI = os.getenv('CI', None) == "true"
 
 try:
     import pytest
@@ -17,9 +25,16 @@ except ImportError:
     USE_PYTEST = False
 
 
+raises: Callable[[Any, Any], Any]
+XFAIL: Callable[[Any], Any]
+skip: Callable[[Any], Any]
+SKIP: Callable[[Any], Any]
+slow: Callable[[Any], Any]
+nocache_fail: Callable[[Any], Any]
+
+
 if USE_PYTEST:
     raises = pytest.raises
-    warns = pytest.warns
     skip = pytest.skip
     XFAIL = pytest.mark.xfail
     SKIP = pytest.mark.skip
@@ -180,39 +195,98 @@ else:
         "Dummy decorator for marking tests that fail when cache is disabled"
         return func
 
-    @contextlib.contextmanager
-    def warns(warningcls, *, match=''):
-        '''Like raises but tests that warnings are emitted.
+@contextlib.contextmanager
+def warns(warningcls, *, match='', test_stacklevel=True):
+    '''
+    Like raises but tests that warnings are emitted.
 
-        >>> from sympy.testing.pytest import warns
-        >>> import warnings
+    >>> from sympy.testing.pytest import warns
+    >>> import warnings
 
-        >>> with warns(UserWarning):
-        ...     warnings.warn('deprecated', UserWarning)
+    >>> with warns(UserWarning):
+    ...     warnings.warn('deprecated', UserWarning, stacklevel=2)
 
-        >>> with warns(UserWarning):
-        ...     pass
-        Traceback (most recent call last):
-        ...
-        Failed: DID NOT WARN. No warnings of type UserWarning\
-        was emitted. The list of emitted warnings is: [].
-        '''
-        # Absorbs all warnings in warnrec
-        with warnings.catch_warnings(record=True) as warnrec:
-            # Hide all warnings but make sure that our warning is emitted
-            warnings.simplefilter("ignore")
-            warnings.filterwarnings("always", match, warningcls)
-            # Now run the test
-            yield
+    >>> with warns(UserWarning):
+    ...     pass
+    Traceback (most recent call last):
+    ...
+    Failed: DID NOT WARN. No warnings of type UserWarning\
+    was emitted. The list of emitted warnings is: [].
 
-        # Raise if expected warning not found
-        if not any(issubclass(w.category, warningcls) for w in warnrec):
-            msg = ('Failed: DID NOT WARN.'
-                   ' No warnings of type %s was emitted.'
-                   ' The list of emitted warnings is: %s.'
-                   ) % (warningcls, [w.message for w in warnrec])
-            raise Failed(msg)
+    ``test_stacklevel`` makes it check that the ``stacklevel`` parameter to
+    ``warn()`` is set so that the warning shows the user line of code (the
+    code under the warns() context manager). Set this to False if this is
+    ambiguous or if the context manager does not test the direct user code
+    that emits the warning.
 
+    If the warning is a ``SymPyDeprecationWarning``, this additionally tests
+    that the ``active_deprecations_target`` is a real target in the
+    ``active-deprecations.md`` file.
+
+    '''
+    # Absorbs all warnings in warnrec
+    with warnings.catch_warnings(record=True) as warnrec:
+        # Any warning other than the one we are looking for is an error
+        warnings.simplefilter("error")
+        warnings.filterwarnings("always", category=warningcls)
+        # Now run the test
+        yield warnrec
+
+    # Raise if expected warning not found
+    if not any(issubclass(w.category, warningcls) for w in warnrec):
+        msg = ('Failed: DID NOT WARN.'
+               ' No warnings of type %s was emitted.'
+               ' The list of emitted warnings is: %s.'
+               ) % (warningcls, [w.message for w in warnrec])
+        raise Failed(msg)
+
+    # We don't include the match in the filter above because it would then
+    # fall to the error filter, so we instead manually check that it matches
+    # here
+    for w in warnrec:
+        # Should always be true due to the filters above
+        assert issubclass(w.category, warningcls)
+        if not re.compile(match, re.I).match(str(w.message)):
+            raise Failed(f"Failed: WRONG MESSAGE. A warning with of the correct category ({warningcls.__name__}) was issued, but it did not match the given match regex ({match!r})")
+
+    if test_stacklevel:
+        for f in inspect.stack():
+            thisfile = f.filename
+            file = os.path.split(thisfile)[1]
+            if file.startswith('test_'):
+                break
+            elif file == 'doctest.py':
+                # skip the stacklevel testing in the doctests of this
+                # function
+                return
+        else:
+            raise RuntimeError("Could not find the file for the given warning to test the stacklevel")
+        for w in warnrec:
+            if w.filename != thisfile:
+                msg = f'''\
+Failed: Warning has the wrong stacklevel. The warning stacklevel needs to be
+set so that the line of code shown in the warning message is user code that
+calls the deprecated code (the current stacklevel is showing code from
+{w.filename} (line {w.lineno}), expected {thisfile})'''.replace('\n', ' ')
+                raise Failed(msg)
+
+    if warningcls == SymPyDeprecationWarning:
+        this_file = pathlib.Path(__file__)
+        active_deprecations_file = (this_file.parent.parent.parent / 'doc' /
+                                    'src' / 'explanation' /
+                                    'active-deprecations.md')
+        if not active_deprecations_file.exists():
+            # We can only test that the active_deprecations_target works if we are
+            # in the git repo.
+            return
+        targets = []
+        for w in warnrec:
+            targets.append(w.message.active_deprecations_target)
+        with open(active_deprecations_file, encoding="utf-8") as f:
+            text = f.read()
+        for target in targets:
+            if f'({target})=' not in text:
+                raise Failed(f"The active deprecations target {target!r} does not appear to be a valid target in the active-deprecations.md file ({active_deprecations_file}).")
 
 def _both_exp_pow(func):
     """
@@ -237,18 +311,31 @@ def _both_exp_pow(func):
 
 @contextlib.contextmanager
 def warns_deprecated_sympy():
-    '''Shorthand for ``warns(SymPyDeprecationWarning)``
+    '''
+    Shorthand for ``warns(SymPyDeprecationWarning)``
 
     This is the recommended way to test that ``SymPyDeprecationWarning`` is
     emitted for deprecated features in SymPy. To test for other warnings use
     ``warns``. To suppress warnings without asserting that they are emitted
     use ``ignore_warnings``.
 
+    .. note::
+
+       ``warns_deprecated_sympy()`` is only intended for internal use in the
+       SymPy test suite to test that a deprecation warning triggers properly.
+       All other code in the SymPy codebase, including documentation examples,
+       should not use deprecated behavior.
+
+       If you are a user of SymPy and you want to disable
+       SymPyDeprecationWarnings, use ``warnings`` filters (see
+       :ref:`silencing-sympy-deprecation-warnings`).
+
     >>> from sympy.testing.pytest import warns_deprecated_sympy
-    >>> from sympy.utilities.exceptions import SymPyDeprecationWarning
+    >>> from sympy.utilities.exceptions import sympy_deprecation_warning
     >>> with warns_deprecated_sympy():
-    ...     SymPyDeprecationWarning("Don't use", feature="old thing",
-    ...         deprecated_since_version="1.0", issue=123).warn()
+    ...     sympy_deprecation_warning("Don't use",
+    ...        deprecated_since_version="1.0",
+    ...        active_deprecations_target="active-deprecations")
 
     >>> with warns_deprecated_sympy():
     ...     pass
@@ -256,52 +343,46 @@ def warns_deprecated_sympy():
     ...
     Failed: DID NOT WARN. No warnings of type \
     SymPyDeprecationWarning was emitted. The list of emitted warnings is: [].
+
+    .. note::
+
+       Sometimes the stacklevel test will fail because the same warning is
+       emitted multiple times. In this case, you can use
+       :func:`sympy.utilities.exceptions.ignore_warnings` in the code to
+       prevent the ``SymPyDeprecationWarning`` from being emitted again
+       recursively. In rare cases it is impossible to have a consistent
+       ``stacklevel`` for deprecation warnings because different ways of
+       calling a function will produce different call stacks.. In those cases,
+       use ``warns(SymPyDeprecationWarning)`` instead.
+
+    See Also
+    ========
+    sympy.utilities.exceptions.SymPyDeprecationWarning
+    sympy.utilities.exceptions.sympy_deprecation_warning
+    sympy.utilities.decorator.deprecated
+
     '''
     with warns(SymPyDeprecationWarning):
         yield
 
-@contextlib.contextmanager
-def ignore_warnings(warningcls):
-    '''Context manager to suppress warnings during tests.
 
-    This function is useful for suppressing warnings during tests. The warns
-    function should be used to assert that a warning is raised. The
-    ignore_warnings function is useful in situation when the warning is not
-    guaranteed to be raised (e.g. on importing a module) or if the warning
-    comes from third-party code.
+def _running_under_pyodide():
+    """Test if running under pyodide."""
+    try:
+        import pyodide_js  # type: ignore  # noqa
+    except ImportError:
+        return False
+    else:
+        return True
 
-    When the warning is coming (reliably) from SymPy the warns function should
-    be preferred to ignore_warnings.
 
-    >>> from sympy.testing.pytest import ignore_warnings
-    >>> import warnings
-
-    Here's a warning:
-
-    >>> with warnings.catch_warnings():  # reset warnings in doctest
-    ...     warnings.simplefilter('error')
-    ...     warnings.warn('deprecated', UserWarning)
-    Traceback (most recent call last):
-      ...
-    UserWarning: deprecated
-
-    Let's suppress it with ignore_warnings:
-
-    >>> with warnings.catch_warnings():  # reset warnings in doctest
-    ...     warnings.simplefilter('error')
-    ...     with ignore_warnings(UserWarning):
-    ...         warnings.warn('deprecated', UserWarning)
-
-    (No warning emitted)
-    '''
-    # Absorbs all warnings in warnrec
-    with warnings.catch_warnings(record=True) as warnrec:
-        # Make sure our warning doesn't get filtered
-        warnings.simplefilter("always", warningcls)
-        # Now run the test
-        yield
-
-    # Reissue any warnings that we aren't testing for
-    for w in warnrec:
-        if not issubclass(w.category, warningcls):
-            warnings.warn_explicit(w.message, w.category, w.filename, w.lineno)
+def skip_under_pyodide(message):
+    """Decorator to skip a test if running under pyodide."""
+    def decorator(test_func):
+        @functools.wraps(test_func)
+        def test_wrapper():
+            if _running_under_pyodide():
+                skip(message)
+            return test_func()
+        return test_wrapper
+    return decorator
